import markdown2
from django import template
from inspect import signature
from rpc.views import _get_rpc_methods

all_methods = _get_rpc_methods()
register = template.Library()


def infer_json_type(py_type: str) -> str:
    py_type = py_type.strip()
    types = {
        "int": "number",
        "float": "number",
        "list": "array",
        "dict": "object",
        "str": "string",
        "None": "null",
        "List[dict]": "array[object]",
        "List[Dict]": "array[object]",
        "bool": "boolean",
        "Union[int, bool]": "number | boolean",
        "Dict[str, List]": "object",
        "List[str]": "array[string]",
        "List[List[dict]]": "array[object]",
        "Optional[Dict]": "object | null",
        "False": "false",
        "True": "true",
        "Dict": "object",
    }
    return types.get(py_type, py_type)


@register.filter(name="doc_string")
def doc_string(fn_name: str) -> str:
    doc = all_methods[fn_name].__doc__
    if not doc:
        return "No documentation available"
    ds = doc.strip()
    return markdown2.markdown(ds)


@register.filter(name="returntype")
def arguments(fn_name):
    sign = signature(all_methods[fn_name])
    _, return_type = str(sign).split("->")
    return infer_json_type(return_type.strip())


@register.filter(name="parameters")
def parameters(fn_name):
    out = ""
    sign = signature(all_methods[fn_name])
    shortened_signature = (
        str(sign)
        .replace("user: django.contrib.auth.models.User,", "")
        .replace("user: django.contrib.auth.models.User ", "")
        .replace("user: django.contrib.auth.models.User", "")
        .replace("NoneType", "None")
        .replace("(", "")
        .replace(")", "")
    ).strip()
    params, _ = shortened_signature.split("->")
    for param in params.split(","):
        param = param.strip()
        if not param:
            continue
        param_name, param_type = param.split(":")

        try:
            param_type, param_default = param_type.split("=")
        except ValueError:
            param_default = None

        out = out + "<li><strong>{}</strong> ({}{})</li>".format(
            param_name.strip(),
            infer_json_type(param_type.strip()),
            (
                " <em> defaults to " + infer_json_type(param_default) + "</em>"
                if param_default
                else ""
            ),
        )
    if not out:
        return "<em>no parameters</em>"
    return "<ul>{}</ul>".format(out)


@register.filter(name="rpc_groups_from_function")
def rpc_groups_from_function(fn_name: str) -> str:
    fn = all_methods[fn_name]
    return " ".join(getattr(fn, "rpc_groups", []))


@register.filter(name="rpc_groups_as_list")
def rpc_groups_as_list(fns) -> list:
    groups = []
    for _, fn in fns:
        grps = getattr(fn, "rpc_groups", [])
        for group in grps:
            groups.append(group)
    return list(set(groups))



from django.core.management.base import BaseCommand
from django.db import connection
from resources.helpers import make_iiif
from concurrent.futures import ProcessPoolExecutor


def tile_file(pk: int):
    try:
        make_iiif(pk, True)
        print(pk)
    except Exception as e:
        print(e)
        raise e


class Command(BaseCommand):
    def handle(self, *args, **options):
        pks = []
        cursor = connection.cursor()
        cursor.execute(
            "select resource_ptr_id from resources_file where resources_file.tiled = false order by resource_ptr_id"
        )
        for row in cursor.fetchall():
            pks.append(row[0])
        connection.close()
        with ProcessPoolExecutor(max_workers=10) as executor:
            for pk in pks:
                executor.submit(tile_file, pk)



# Generated by Django 3.2.2 on 2021-10-20 09:48

from django.db import migrations


def fixtures(*args):
    pass
    # Permission.objects.get_or_create(label="collection.public_only") # no more needed


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0022_auto_20211020_0844"),
    ]

    operations = [migrations.RunPython(fixtures)]



from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from django.http import HttpRequest, HttpResponse
from rpc import methods as jama


@login_required
def index(request: HttpRequest) -> HttpResponse:
    accesses = jama.projects_user_permissions(request.user)
    for access in accesses:
        access["project_stats"] = jama.project_stats(
            request.user, access["project"]["id"]
        )
    return render(
        request,
        "ui/index.html",
        {"accesses": accesses, "choose_project_page": True},
    )



from django.http import HttpResponse, HttpRequest
from django.shortcuts import render, get_object_or_404, redirect
from jama import settings
from django.contrib.auth.models import User
from resources.models import Resource, Collection

BASE_PATH = "/" + settings.JAMA_URL_BASE_PATH


def handler404(request, *args, **argv):
    print(request.path)
    return HttpResponse("", status=404)


def homepage(request: HttpRequest) -> HttpResponse:
    return render(request, "homepage.html", {"BASE_PATH": BASE_PATH})


def status(request: HttpRequest) -> HttpResponse:
    # testing database connection
    try:
        User.objects.first()
    except:  # noqa: E722
        return HttpResponse("ko")
    return HttpResponse("ok")


def ark_resource(request: HttpRequest, resource_id: int) -> HttpResponse:
    resource = get_object_or_404(Resource, pk=resource_id, deleted_at__isnull=True)
    if resource.ptr_project.ark_redirect:
        location = (
            resource.ptr_project.ark_redirect.replace("[CLASS]", "resource")
            .replace("[ARK]", str(resource.ark))
            .replace("[PK]", str(resource.pk))
        )
        return redirect(location)
    return HttpResponse("Jama Resource ARK location placeholder")


def ark_collection(request: HttpRequest, collection_id: int) -> HttpResponse:
    collection = get_object_or_404(
        Collection, pk=collection_id, deleted_at__isnull=True, public_access=True
    )
    if collection.project.ark_redirect:
        location = (
            collection.project.ark_redirect.replace("[CLASS]", "collection")
            .replace("[ARK]", str(collection.ark))
            .replace("[PK]", str(collection.pk))
        )
        return redirect(location)
    return HttpResponse("Jama Collection ARK location placeholder")



from django.core.management.base import BaseCommand, CommandError
from resources.models import Project, File
import os
from django.utils.text import slugify


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("destination_dir", type=str)

    def handle(self, *args, **options):
        destination_dir = options.get("destination_dir").rstrip("/")
        if not os.path.isdir(destination_dir):
            raise CommandError("{} is not a directory".format(destination_dir))

        for project in Project.objects.all():
            for f in File.objects.filter(project=project):
                for col in f.collections.all():
                    normalized_names = [
                        destination_dir,
                        "{}-{}".format(project.label, project.pk),
                    ]
                    for dir in col.to_path():
                        normalized_names.append(slugify(dir))
                    full_dir_path = "/".join(normalized_names)
                    fname, _ = os.path.splitext(f.title)
                    full_file_path = (
                        os.path.join(full_dir_path, slugify(fname))
                        + "."
                        + f.file_type.extensions.first().label
                    )
                    os.makedirs(full_dir_path, exist_ok=True)
                    try:
                        os.symlink(f.local_path(), full_file_path)
                    except FileExistsError:
                        pass
                    print(full_file_path)



# Generated by Django 4.0 on 2022-01-21 10:25

from django.db import migrations

# from resources import models


def add_fixtures(apps, schema_editor):
    Permission = apps.get_model("resources", "Permission")
    Project = apps.get_model("resources", "Project")
    Role = apps.get_model("resources", "Project")
    create_right, _ = Permission.objects.get_or_create(label="project_property.create")
    read_right, _ = Permission.objects.get_or_create(label="project_property.read")
    update_right, _ = Permission.objects.get_or_create(label="project_property.update")
    delete_right, _ = Permission.objects.get_or_create(label="project_property.delete")
    for project in Project.objects.all():
        admin_role = Role.objects.filter(project=project, label="admin").first()
        if admin_role:
            admin_role.permissions.add(create_right)
            admin_role.permissions.add(read_right)
            admin_role.permissions.add(update_right)
            admin_role.permissions.add(delete_right)


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0025_projectproperty"),
    ]

    operations = [migrations.RunPython(add_fixtures)]



from django.core.management.base import BaseCommand
from resources.models import File
from resources.tasks import iiif_task
from resources.helpers import iiif_destination_dir_from_hash
import os
from django.utils import timezone


class Command(BaseCommand):
    def handle(self, *args, **options):
        start_date = timezone.now() - timezone.timedelta(days=1)
        for f in File.objects.filter(
            file_type__serve_with_iiif=True,
            deleted_at__isnull=True,
            created_at__gte=start_date,
        ).distinct():
            iiif_destination_dir = iiif_destination_dir_from_hash(f.hash)
            iiif_destination_file = "{}{}".format(iiif_destination_dir, f.hash)
            if not os.path.exists(iiif_destination_file):
                iiif_task(f.pk)



from django.core.management.base import BaseCommand, CommandError
import os
from django.contrib.auth.models import User
from typing import Iterator
from rpc.methods import add_collection_from_path, add_resource_to_collection
from functools import lru_cache
from resources.helpers import handle_local_file
from resources.models import Project, FileExtension
from resources import tasks
from concurrent.futures import ThreadPoolExecutor
import multiprocessing
from pathlib import Path


@lru_cache(maxsize=None)
def cached_add_collection_from_path(*args, **kwargs):
    return add_collection_from_path(*args, **kwargs)


ALLOWED_EXTENSIONS = []

for ext in FileExtension.objects.all():
    ALLOWED_EXTENSIONS.append(".{}".format(ext.label))


def scan_dir(start_path: str, extension: str = None) -> Iterator[str]:
    for root, dirs, files in os.walk(start_path):
        for file in files:
            if extension:
                if file[(len(extension) * -1) :].lower() == extension.lower():
                    yield os.path.join(root, file)
            else:
                yield os.path.join(root, file)


def _add_file(
    file_path: str,
    project: Project,
    current_user: User,
    start_dir: str,
    delete_source: bool = False,
):
    resource_id = handle_local_file(file_path, project)
    tasks.iiif_task(resource_id)
    tasks.exif_task(resource_id)
    tasks.ocr_task(resource_id)
    collection_path = os.path.dirname(file_path)[len(start_dir) :]
    if collection_path:
        hierarchy_of_collections = cached_add_collection_from_path(
            current_user, collection_path, project.pk
        )
        collection_id = hierarchy_of_collections[-1]["id"]
        if add_resource_to_collection(current_user, resource_id, collection_id):
            if delete_source:
                Path(file_path).unlink(missing_ok=True)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("start_dir", type=str)
        parser.add_argument("user", type=str)
        parser.add_argument("project", type=int)
        parser.add_argument("--extensions", nargs="+", type=str)
        parser.add_argument("--delete", action="store_true")

    def handle(self, *args, **options):
        if options.get("extensions"):
            allowed_extensions = options.get("extensions")
        else:
            allowed_extensions = ALLOWED_EXTENSIONS
        delete_source = options.get("delete", False)
        start_dir = options.get("start_dir")
        if not os.path.isdir(start_dir):
            raise CommandError("{} is not a directory".format(start_dir))
        try:
            project = Project.objects.get(pk=options.get("project"))
        except Project.DoesNotExist:
            raise CommandError(
                "({}) is not a known project".format(options.get("project"))
            )
        try:
            current_user = User.objects.get(username=options.get("user"))
        except User.DoesNotExist:
            raise CommandError("({}) is not a known user".format(options.get("user")))

        executor = ThreadPoolExecutor(max_workers=multiprocessing.cpu_count())

        for file_path in scan_dir(start_dir):
            _, extension = os.path.splitext(file_path)
            if extension.lower() not in allowed_extensions:
                continue
            executor.submit(
                _add_file, file_path, project, current_user, start_dir, delete_source
            )



# Generated by Django 5.0.1 on 2024-02-16 07:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0030_file_denormalized_image_height_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="file",
            name="text_boxes",
            field=models.JSONField(
                blank=True, null=True, verbose_name="text boxes de tesseract"
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-11-08 08:42

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0036_alter_collection_options_alter_resource_options"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="collection",
            name="is_oai_record",
        ),
    ]



# Generated by Django 5.1.3 on 2024-11-28 13:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0037_remove_collection_is_oai_record"),
    ]

    operations = [
        migrations.AddField(
            model_name="file",
            name="tiled",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 4.2.5 on 2023-10-04 08:51

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0027_auto_20220131_1636"),
    ]

    operations = [
        migrations.AddField(
            model_name="collection",
            name="ark",
            field=models.CharField(max_length=64, null=True, verbose_name="nom ARK"),
        ),
        migrations.AddField(
            model_name="resource",
            name="ark",
            field=models.CharField(max_length=64, null=True, verbose_name="nom ARK"),
        ),
    ]



# Generated by Django 3.2.2 on 2021-09-21 09:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0018_alter_project_label"),
    ]

    operations = [
        migrations.AlterField(
            model_name="collection",
            name="project",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="resources.project",
            ),
        ),
    ]



from resources import models
from jama import settings
import os
from rpc.cache import SerializerCache


def get_cache(cache: SerializerCache = None):
    if not cache:
        return SerializerCache()
    return cache


def iiif_file_exists(file_hash) -> bool:
    path_to_test = "{}/{}".format(
        settings.IIIF_DIR, models.hash_to_iiif_path(file_hash)
    )
    return os.path.isfile(path_to_test)


def file_type(ftype: models.FileType, cache: SerializerCache = None) -> dict:
    extensions = []
    for ext in ftype.extensions.all():
        extensions.append(ext.label)
    return {
        "mime": ftype.mime,
        "extensions": extensions,
        "iiif_support": ftype.serve_with_iiif,
    }


def metadataset(
    metadataset_instance: models.MetadataSet, cache: SerializerCache = None
) -> dict:
    return {
        "object_type": "metadataset",
        "id": metadataset_instance.id,
        "title": metadataset_instance.title,
        "project_id": metadataset_instance.project_id,
        "metas_count": metadataset_instance.metadata_set.filter(expose=True).count(),
    }


def metadata(metadata_instance: models.Metadata, cache: SerializerCache = None) -> dict:
    set_instance = get_cache(cache).fetch_obj("metadataset", metadata_instance.set_id)
    return {
        "object_type": "metadata",
        "id": metadata_instance.id,
        "title": metadata_instance.title,
        "set_id": metadata_instance.set_id,
        "set_title": set_instance.title,
        "rank": metadata_instance.rank,
        "project_id": metadata_instance.project_id,
    }


def metadata_collection_value(
    metadata_collection_value_instance: models.MetadataCollectionValue,
    cache: SerializerCache = None,
) -> dict:
    metadata_instance = get_cache(cache).fetch_obj(
        "metadata", metadata_collection_value_instance.metadata_id
    )
    return {
        "object_type": "metadata_collection_value",
        "id": metadata_collection_value_instance.id,
        "meta": metadata(metadata_instance, cache=cache),
        "value": metadata_collection_value_instance.value,
    }


def metadata_resource_value(
    metadata_resource_value_instance: models.MetadataResourceValue,
    cache: SerializerCache = None,
) -> dict:
    metadata_instance = get_cache(cache).fetch_obj(
        "metadata", metadata_resource_value_instance.metadata_id
    )
    return {
        "object_type": "metadata_resource_value",
        "id": metadata_resource_value_instance.id,
        "meta": metadata(metadata_instance, cache=cache),
        "value": metadata_resource_value_instance.value,
    }


def collection(
    collection_instance: models.Collection,
    recursive: bool = False,
    only_published: bool = False,
    cache: SerializerCache = None,
) -> dict:
    tags = []
    for tag_instance in collection_instance.tags.all():
        tags.append(tag(tag_instance, cache=cache))
    payload = {
        "object_type": "collection",
        "id": collection_instance.id,
        "title": collection_instance.title,
        "resources_count": collection_instance.resources.filter(
            deleted_at__isnull=True
        ).count(),
        "children_count": collection_instance.children()
        .filter(deleted_at__isnull=True)
        .count(),
        "parent": collection_instance.parent_id,
        "children": None,
        "project_id": collection_instance.project_id,
        "metas": [],
        "public_access": collection_instance.public_access,
        "tags": tags,
        "represented_by": (
            resource(collection_instance.representative, False, cache=cache)
            if collection_instance.representative
            and collection_instance.representative.deleted_at is None
            else None
        ),
        "created_at": collection_instance.created_at.isoformat(),
        "updated_at": collection_instance.updated_at.isoformat(),
        "ark": collection_instance.ark,
        "ancestors": [
            {"id": x.pk, "title": x.title} for x in collection_instance.ancestors()
        ][1:],
    }
    if recursive:
        payload["children"] = []
        for child in collection_instance.children():
            # filter private content
            if only_published and child.public_access is False:
                continue
            payload["children"].append(collection(child, recursive, cache=cache))

    for prop in collection_instance.metadatacollectionvalue_set.all():
        payload["metas"].append(metadata_collection_value(prop, cache=cache))

    return payload


def resource(
    resource_instance: models.Resource,
    include_metas: bool = True,
    cache: SerializerCache = None,
) -> dict:
    try:
        if resource_instance.file:
            return file(resource_instance.file, include_metas, cache=cache)
    except models.File.DoesNotExist:
        pass
    tags = []
    for tag_instance in resource_instance.tags.all():
        tags.append(tag(tag_instance, cache=cache))
    return {
        "object_type": "resource",
        "id": resource_instance.id,
        "title": resource_instance.title,
        "tags": tags,
        "created_at": resource_instance.created_at.isoformat(),
        "updated_at": resource_instance.updated_at.isoformat(),
        "ark": resource_instance.ark,
        "project_id": resource_instance.ptr_project_id,
    }


def file(
    file_instance: models.File,
    include_metas: bool = True,
    cache: SerializerCache = None,
) -> dict:
    props = []
    urls = {
        "download": "{}rpc/download/{}".format(settings.JAMA_SITE, file_instance.pk),
        "download_public": "{}rpc/download_public/{}".format(
            settings.JAMA_SITE, file_instance.hash
        ),
    }
    tags = []
    for tag_instance in file_instance.tags.all():
        tags.append(tag(tag_instance, cache=cache))
    if (
        file_instance.should_have_iiif()
    ):  # and iiif_file_exists(file_instance.hash): # FS is slow
        iiif_path = models.hash_to_iiif_path(
            file_instance.hash, settings.IIIF_PATH_SEPARATOR
        )
        urls["iiif"] = "{}{}/info.json".format(settings.JAMA_IIIF_ENDPOINT, iiif_path)
        urls["s"] = "{}{}/full/{}100,/0/default.jpg".format(
            settings.JAMA_IIIF_ENDPOINT, iiif_path, settings.JAMA_IIIF_UPSCALING_PREFIX
        )
        urls["m"] = "{}{}/full/{}300,/0/default.jpg".format(
            settings.JAMA_IIIF_ENDPOINT, iiif_path, settings.JAMA_IIIF_UPSCALING_PREFIX
        )
        urls["l"] = "{}{}/full/{}1000,/0/default.jpg".format(
            settings.JAMA_IIIF_ENDPOINT, iiif_path, settings.JAMA_IIIF_UPSCALING_PREFIX
        )
        urls["xl"] = "{}{}/full/{}5000,/0/default.jpg".format(
            settings.JAMA_IIIF_ENDPOINT, iiif_path, settings.JAMA_IIIF_UPSCALING_PREFIX
        )
    if include_metas:
        for prop in file_instance.metadataresourcevalue_set.filter(
            metadata__expose=True
        ).order_by("-metadata__set", "metadata__rank", "id"):
            props.append(metadata_resource_value(prop, cache=cache))
    file_type_instance = get_cache(cache).fetch_obj(
        "file_type", file_instance.file_type_id
    )
    return {
        "object_type": "resource",
        "id": file_instance.id,
        "title": file_instance.title,
        "original_name": file_instance.original_name,
        "type": str(file_type_instance),
        "hash": file_instance.hash,
        "metas": props or None,
        "urls": urls,
        "tags": tags,
        "created_at": file_instance.created_at.isoformat(),
        "updated_at": file_instance.updated_at.isoformat(),
        "project_id": file_instance.project.id,
        "ark": file_instance.ark,
        "width": file_instance.denormalized_image_width,
        "height": file_instance.denormalized_image_height,
    }


def metadatatype(
    metada_type: models.MetadataType, cache: SerializerCache = None
) -> dict:
    return {
        "object_type": "metada_type",
        "id": metada_type.id,
        "title": metada_type.title,
    }


def tag(tag_instance: models.Tag, cache: SerializerCache = None) -> dict:
    return {
        "object_type": "tag",
        "id": tag_instance.id,
        "uid": tag_instance.uid,
        "label": tag_instance.label or tag_instance.uid,
        "ark": tag_instance.ark,
    }


def project(project_instance: models.Project, cache: SerializerCache = None) -> dict:
    return {
        "object_type": "project",
        "id": project_instance.pk,
        "label": project_instance.label,
        "description": project_instance.description,
    }


def permission(
    permission_instance: models.Permission, cache: SerializerCache = None
) -> dict:
    return {
        "object_type": "permission",
        "id": permission_instance.pk,
        "label": permission_instance.label,
    }


def role(role_instance: models.Role, cache: SerializerCache = None) -> dict:
    permissions = []
    for permission_instance in role_instance.permissions.all():
        permissions.append(permission(permission_instance, cache=cache))
    return {
        "object_type": "role",
        "id": role_instance.id,
        "label": role_instance.label,
        "permissions": permissions,
    }


def project_access(
    project_access_instance: models.ProjectAccess, cache: SerializerCache = None
) -> dict:
    cache = get_cache(cache)
    role_instance = cache.fetch_obj("role", project_access_instance.role_id)
    project_instance = cache.fetch_obj("project", project_access_instance.project_id)
    user_instance = cache.fetch_obj("user", project_access_instance.user_id)
    return {
        "object_type": "project_access",
        "project": project(project_instance, cache=cache),
        "role": role(role_instance, cache=cache),
        "user": user_instance.username,
    }


def project_property(
    project_property_instance: models.ProjectProperty, cache: SerializerCache = None
) -> dict:
    return {
        "project_id": project_property_instance.project_id,
        "key": project_property_instance.key,
        "value": project_property_instance.value,
    }


def user_task(user_task_instance: models.UserTask) -> dict:
    return {
        "object_type": "task",
        "id": user_task_instance.id,
        "description": user_task_instance.description,
        "created_at": (
            user_task_instance.created_at.isoformat()
            if user_task_instance.created_at
            else None
        ),
        "started_at": (
            user_task_instance.started_at.isoformat()
            if user_task_instance.started_at
            else None
        ),
        "finished_at": (
            user_task_instance.finished_at.isoformat()
            if user_task_instance.finished_at
            else None
        ),
        "failed_at": (
            user_task_instance.failed_at.isoformat()
            if user_task_instance.failed_at
            else None
        ),
        "project_id": user_task_instance.project_id,
    }



CAN_ONLY_SHOW_PUBLIC_COLLECTIONS_CONTENT = "can only show public collections content"
CANT_CREATE_IMAGE = "Can't create image"
COLLECTION_ALREADY_EXIST_IN_PARENT = (
    "collection with same name already exists in parent collection"
)
INVALID_PROPERTY_KEY = "invalid property key"
NEED_COLLECTION_ID_OR_PROJECT_ID = "must give one of collection_id or project_id"
NEED_PROJECT_ID_IF_PARENT_ID_NULL = "project_id is needed when parent_id is null"
NEED_PROJECT_ID_OR_PARENT_ID = "use project_id OR parent_id"
NO_FOOTGUNS = "Can't shoot your own foot"
NO_METADATASET_MIX = "can't mix metadata sets when setting multiple meta values at once"
NO_PATH = "path argument is missing"
NO_PROJECT_ACCESS = "No access ({}) for project {}"
NO_PROJECT_ID = "project_id is not set"
NO_PROJECT_ROOT = "project has no root"
NO_SUCH_COLLECTION = "no such collection"
NO_SUCH_METADATA = "no such metadata"
NO_SUCH_METADATASET = "no such metadatas set"
NO_SUCH_METADATATYPE = "no such metadatatype"
NO_SUCH_PROJECT = "no such project"
NO_SUCH_PROJECT_PROPERTY = "project has no such property"
NO_SUCH_RESOURCE = "no such resource"
NO_SUCH_TAG = "no such tag"
NO_SUCH_USER = "no such user"
NOT_A_FILE = "not a file"
NOT_AN_IMAGE = "not an image"
NOTHING_TO_DO = "nothing to do"
PERM_COLLECTION_CREATE = "collection.create"
PERM_COLLECTION_DELETE = "collection.delete"
PERM_COLLECTION_READ = "collection.read"
PERM_COLLECTION_UPDATE = "collection.update"
PERM_METADATA_CREATE = "metadata.create"
PERM_METADATA_DELETE = "metadata.delete"
PERM_METADATA_READ = "metadata.read"
PERM_METADATA_UPDATE = "metadata.update"
PERM_METADATASET_CREATE = "metadataset.create"
PERM_METADATASET_DELETE = "metadataset.delete"
PERM_METADATASET_READ = "metadataset.read"
PERM_PERMISSION_UPDATE = "permission.update"
PERM_PROJECT_PROPERTY_CREATE = "project_property.create"
PERM_PROJECT_PROPERTY_DELETE = "project_property.delete"
PERM_PROJECT_PROPERTY_READ = "project_property.read"
PERM_PROJECT_PROPERTY_UPDATE = "project_property.update"
PERM_RESOURCE_DELETE = "resource.delete"
PERM_RESOURCE_READ = "resource.read"
PERM_RESOURCE_UPDATE = "resource.update"
PERM_ROLE_READ = "role.read"
PERM_TAG_DELETE = "tag.delete"
PROJECT_MISMATCH = "Can't mix items from different collections"
ROOT_COLLECTION_TITLE = "root"
SEARCH_TERMS_LIMIT = 10
SUPERUSER_NEEDED = "only for superusers"
TOO_MANY_SEARCH_TERMS = "Too many search terms."
UNKNOWN_ERROR = "unknown error"
WRONG_ARGUMENT = "wrong argument"



# Generated by Django 3.2.2 on 2021-09-20 17:27

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0016_auto_20210920_1717"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="resource",
            name="ptr_owner",
        ),
        migrations.AlterUniqueTogether(
            name="collection",
            unique_together={("title", "project", "parent")},
        ),
        migrations.AlterUniqueTogether(
            name="file",
            unique_together={("project", "hash")},
        ),
        migrations.AlterUniqueTogether(
            name="metadata",
            unique_together={("title", "set", "project")},
        ),
        migrations.AlterUniqueTogether(
            name="metadataset",
            unique_together={("title", "project")},
        ),
        migrations.AlterUniqueTogether(
            name="tag",
            unique_together={("uid", "project")},
        ),
        migrations.RemoveField(
            model_name="collection",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="file",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="metadata",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="metadataset",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="tag",
            name="owner",
        ),
    ]



from django.core.management.base import BaseCommand
from inspect import signature
from rpc.views import _get_rpc_methods

all_methods = _get_rpc_methods()

TPL = '''
@_rpc_groups({})
def jama_{}{}:
    """
    {}
    """
    _jama_api_key = APIKey.objects.filter(user=user).first()
    try:
        jama = JamaClient(settings.ATLAS_JAMA_RPC_ENDPOINT, _jama_api_key.key)
        return jama.{}({})
    except JamaServiceError as e:
        raise ServiceException(e.message)
'''


class Command(BaseCommand):
    def handle(self, *args, **options):
        for key in sorted(all_methods.keys()):
            sign = signature(all_methods.get(key))
            keep_params = []
            for param in list(sign.parameters.items())[1:]:
                keep_params.append(param[0])
            shortened_signature = (
                str(sign)
                .replace("user: django.contrib.auth.models.User", "user: User")
                .replace("NoneType", "None")
            )
            print(
                TPL.format(
                    all_methods.get(key).rpc_groups,
                    key,
                    shortened_signature,
                    all_methods.get(key).__doc__.strip(),
                    key,
                    ", ".join(keep_params),
                )
            )



import os
from django.core.management.base import BaseCommand, CommandError
from fuse import FUSE, FuseOSError, Operations
import errno
from resources.models import Collection, Resource, slugify, File
from jama import settings
from functools import lru_cache
import time


def get_ttl_hash(seconds=3600):
    """Return the same value withing `seconds` time period"""
    return round(time.time() / seconds)


@lru_cache(maxsize=1024)
def _db_obj_from_path(path: str, ttl_hash=None):
    """
    For fast access, paths should include objects pk:

    ex_collection-2/other_col-23/more_col-43/some_pic-374.jpg
    """
    last_item = path.rstrip("/").split("/")[-1]
    if last_item:
        name, ext = os.path.splitext(last_item)
        pk = name.split("-")[-1]
        try:
            pk = int(pk)
        except ValueError:
            return
        if not ext:
            return Collection.objects.filter(pk=pk).first()
        else:
            return Resource.objects.filter(pk=pk).first()
    return


class Ops(Operations):
    def __init__(self, root_collection: Collection):
        self.root_collection = root_collection

    def _obj_from_path(self, path: str):
        """
        For fast access, paths should include objects pk:

        ex_collection-2/other_col-23/more_col-43/some_pic-374.jpg
        """
        if path == "/":
            return self.root_collection
        obj = _db_obj_from_path(path, ttl_hash=get_ttl_hash())
        if not obj:
            raise FuseOSError(errno.ENOENT)
        else:
            return obj

    def _collection_fs_name(self, collection: Collection) -> str:
        return f"{slugify(collection.title)}-{collection.pk}"

    def _file_fs_name(self, file: File) -> str:
        return (
            f"{slugify(file.title)}-{file.pk}.{file.file_type.extensions.first().label}"
        )

    # Filesystem methods

    def access(self, path, mode):
        """
        modes:

        0 -> no access
        1 -> execution
        2 -> write
        4 -> read
        """
        # we want a read-only file system,
        # with execution for dirs
        if mode in [2]:
            raise FuseOSError(errno.EACCES)
        obj = self._obj_from_path(path)
        if type(obj) is Resource and mode == 1:
            raise FuseOSError(errno.EACCES)

    def chmod(self, path, mode):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def chown(self, path, uid, gid):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def getattr(self, path, fh=None):
        obj = self._obj_from_path(path=path)
        if type(obj) is Resource:
            if obj.file:
                st = os.lstat(obj.file.local_path())
                return dict(
                    (key, getattr(st, key))
                    for key in (
                        "st_atime",
                        "st_ctime",
                        "st_gid",
                        "st_mode",
                        "st_mtime",
                        "st_nlink",
                        "st_size",
                        "st_uid",
                    )
                )
        if type(obj) is Collection:
            st = os.lstat(settings.MEDIA_FILES_DIR)
            st_dict = dict(
                (key, getattr(st, key))
                for key in (
                    "st_atime",
                    "st_ctime",
                    "st_gid",
                    "st_mode",
                    "st_mtime",
                    "st_nlink",
                    "st_size",
                    "st_uid",
                )
            )
            return st_dict
        raise FuseOSError(errno.ENOENT)

    def readdir(self, path: str, fh):
        collection: Collection = self._obj_from_path(path=path)
        for res in collection.resources.filter(deleted_at__isnull=True):
            if res.file:
                yield self._file_fs_name(res.file)
        for col in collection.children().filter(deleted_at__isnull=True):
            yield self._collection_fs_name(col)

    def readlink(self, path):
        # no symlinks in Jama
        raise FuseOSError(errno.EACCES)

    def mknod(self, path, mode, dev):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def rmdir(self, path):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def mkdir(self, path, mode):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def statfs(self, path):
        # Using MEDIA_FILES_DIR as all Jama File resources are located there
        stv = os.statvfs(settings.MEDIA_FILES_DIR)
        return dict(
            (key, getattr(stv, key))
            for key in (
                "f_bavail",
                "f_bfree",
                "f_blocks",
                "f_bsize",
                "f_favail",
                "f_ffree",
                "f_files",
                "f_flag",
                "f_frsize",
                "f_namemax",
            )
        )

    def unlink(self, path):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def symlink(self, name, target):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def rename(self, old, new):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def link(self, target, name):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def utimens(self, path, times=None):
        raise FuseOSError(errno.EACCES)

    # File methods
    # ============

    def open(self, path, flags):
        if flags != os.O_RDONLY:
            raise FuseOSError(errno.EACCES)
        f = self._obj_from_path(path=path)
        if not f:
            raise FuseOSError(errno.ENOENT)
        if type(f) is not Resource:
            raise FuseOSError(errno.ENOENT)
        if not f.file:
            raise FuseOSError(errno.ENOENT)
        return os.open(f.file.local_path(), os.O_RDONLY)

    def create(self, path, mode, fi=None):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def read(self, path, length, offset, fh):
        os.lseek(fh, offset, os.SEEK_SET)
        return os.read(fh, length)

    def write(self, path, buf, offset, fh):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def truncate(self, path, length, fh=None):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def flush(self, path, fh):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def release(self, path, fh):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)

    def fsync(self, path, fdatasync, fh):
        # no use in read-only fs
        raise FuseOSError(errno.EACCES)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("mount_point", type=str)
        parser.add_argument("root_collection", type=int)

    def handle(self, *args, **options):
        mount_point = options.get("mount_point")
        if not os.path.isdir(mount_point):
            raise CommandError(f"{mount_point} is not a directory")
        root_collection_id = options.get("root_collection")
        root_collection = Collection.objects.filter(pk=root_collection_id).first()
        if not root_collection:
            raise CommandError(f"{root_collection_id} is not a valid collection id")
        FUSE(
            Ops(root_collection),
            mount_point,
            nothreads=True,
            foreground=True,
            allow_other=True,
        )



# Generated by Django 3.2.2 on 2021-10-07 17:48

from django.db import migrations
from resources.models import FileType, FileExtension


def fixtures(*args):
    f_type, _ = FileType.objects.get_or_create(title="prout", mime="application/prout")
    f_ext, _ = FileExtension.objects.get_or_create(label="prout")
    f_type.extensions.add(f_ext)


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0020_project_admin_mail"),
    ]

    operations = [migrations.RunPython(fixtures)]



# Generated by Django 5.0.1 on 2024-04-14 13:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0034_usertask_project"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="file",
            name="post_treated",
        ),
        migrations.AddField(
            model_name="project",
            name="use_exiftool",
            field=models.BooleanField(default=True),
        ),
    ]



from django.http import (
    HttpRequest,
    HttpResponse,
    Http404,
)
from django.template.loader import render_to_string
from jama import settings
from resources.models import Collection, Project
from datetime import datetime
from django.views.decorators.csrf import csrf_exempt
from django.core.paginator import Paginator

PAGINATION_SIZE = 100


def _set_spec_to_collection(set_spec: str) -> Collection:
    try:
        last_id = int(set_spec.split(":")[-1])
        collection = Collection.objects.get(pk=last_id)
        return collection
    except (ValueError, Collection.DoesNotExist):
        raise ValueError("no such collection")


def _collection_to_set_spec(collection: Collection) -> str:
    return collection.oai_setspec()


def _validate_date(in_date: str) -> bool:
    try:
        res = bool(datetime.strptime(in_date, "%Y-%m-%d"))
    except (ValueError, TypeError):
        res = False
    return res


OAI_VERBS = [
    "Identify",
    "GetRecord",
    "ListIdentifiers",
    "ListMetadataFormats",
    "ListRecords",
    "ListSets",
]


def _oai_error(project: Project, request: HttpRequest, errors: dict) -> HttpResponse:
    return HttpResponse(
        render_to_string(
            "oai/errors.xml",
            {"project": project, "base_url": settings.JAMA_SITE, "errors": errors},
        ),
        content_type="text/xml",
    )


def _get_record(project: Project, request: HttpRequest) -> HttpResponse:
    try:
        identifier = int(request.GET.get("identifier")) or request.POST.get(
            "identifier"
        )
        if not identifier:
            return _oai_error(
                project, request, {"badArgument": "identifier is missing"}
            )
    except ValueError:
        return _oai_error(
            project, request, {"idDoesNotExist": "identifier does not exist"}
        )
    try:
        collection_instance = Collection.objects.get(pk=identifier, project=project)
        return HttpResponse(
            render_to_string(
                "oai/get_record.xml",
                {
                    "project": project,
                    "base_url": settings.JAMA_SITE,
                    "collection": collection_instance,
                },
            ),
            content_type="text/xml",
        )
    except Collection.DoesNotExist:
        return _oai_error(
            project, request, {"idDoesNotExist": "identifier does not exist"}
        )


def _identify(project: Project, request: HttpRequest) -> HttpResponse:
    earliest_record = (
        Collection.objects.filter(project=project, public_access=True)
        .order_by("created_at")
        .first()
    )
    return HttpResponse(
        render_to_string(
            "oai/identify.xml",
            {
                "project": project,
                "base_url": settings.JAMA_SITE,
                "earliest_record": earliest_record,
            },
        ),
        content_type="text/xml",
    )


def _list_identifiers(project: Project, request: HttpRequest) -> HttpResponse:
    return _list_records(project, request, only_headers=True)


def _list_metadata_formats(project: Project, request: HttpRequest) -> HttpResponse:
    return HttpResponse(
        render_to_string(
            "oai/list_metadata_formats.xml",
            {
                "project": project,
                "base_url": settings.JAMA_SITE,
            },
        ),
        content_type="text/xml",
    )


def _list_records(
    project: Project, request: HttpRequest, only_headers=False
) -> HttpResponse:
    page_number = 1
    resumption_token = request.GET.get("resumptionToken") or request.POST.get(
        "resumptionToken"
    )

    #
    # Get page number (ie. resumption token)
    #
    if resumption_token:
        try:
            page_number = abs(int(resumption_token))
        except ValueError:
            pass

    #
    # Get from date
    #
    from_date = request.GET.get("from") or request.POST.get("until")
    if from_date and not _validate_date(from_date):
        return _oai_error(project, request, {"badArgument": "bad from date format"})

    #
    # Get until date
    #
    until_date = request.GET.get("until") or request.POST.get("until")
    if until_date and not _validate_date(until_date):
        return _oai_error(project, request, {"badArgument": "bad until date format"})

    #
    # Get setSpec collection
    #
    spec_collection = None
    set_spec = request.GET.get("set") or request.POST.get("set")
    if set_spec:
        spec_collection = _set_spec_to_collection(set_spec=set_spec)

    #
    # Build query
    #
    all_records = Collection.objects.filter(
        project=project, public_access=True
    ).order_by("created_at")
    if from_date:
        all_records = all_records.filter(created_at__gte=from_date)
    if until_date:
        all_records = all_records.filter(created_at__lte=until_date)
    if spec_collection:
        all_records = all_records.filter(
            parent__in=spec_collection.descendants_and_self_ids()
        )

    #
    # Paginate, render.
    #
    total_count = all_records.count()
    if total_count == 0:
        return _oai_error(project, request, {"noRecordsMatch": "no records match"})
    paginator = Paginator(all_records, PAGINATION_SIZE)
    page = paginator.page(page_number)
    if only_headers:
        xml_template = "oai/list_identifiers.xml"
    else:
        xml_template = "oai/list_records.xml"
    return HttpResponse(
        render_to_string(
            xml_template,
            {
                "project": project,
                "base_url": settings.JAMA_SITE,
                "page": page,
                "from": from_date,
                "until": until_date,
                "total_count": total_count,
                "total_pages": paginator.num_pages,
                "nb_of_already_delivered_identifiers": (page_number - 1)
                * PAGINATION_SIZE,
                "spec_collection": spec_collection,
            },
        ),
        content_type="text/xml",
    )


def _list_sets(project: Project, request: HttpRequest) -> HttpResponse:
    collections = Collection.objects.filter(project=project, public_access=True)
    return HttpResponse(
        render_to_string(
            "oai/list_sets.xml",
            {
                "project": project,
                "base_url": settings.JAMA_SITE,
                "collections": collections,
            },
        ),
        content_type="text/xml",
    )


@csrf_exempt
def oai(request: HttpRequest, project_id: int) -> HttpResponse:
    try:
        project = Project.objects.get(pk=project_id)
        oai_verb = request.GET.get("verb") or request.POST.get("verb")
        if oai_verb not in OAI_VERBS:
            return _oai_error(project, request, {"badVerb": "Illegal OAI verb"})
        metadata_prefix = request.GET.get("metadataPrefix") or request.POST.get(
            "metadataPrefix"
        )
        if metadata_prefix and metadata_prefix != "oai_dc":
            return _oai_error(
                project, request, {"cannotDisseminateFormat": "oai_dc only"}
            )
        if oai_verb == "GetRecord":
            return _get_record(project, request)
        if oai_verb == "Identify":
            return _identify(project, request)
        if oai_verb == "ListIdentifiers":
            return _list_identifiers(project, request)
        if oai_verb == "ListMetadataFormats":
            return _list_metadata_formats(project, request)
        if oai_verb == "ListRecords":
            return _list_records(project, request)
        if oai_verb == "ListSets":
            return _list_sets(project, request)
    # each project has its own endpoint
    except Project.DoesNotExist:
        raise Http404()



# Generated by Django 3.2.2 on 2021-05-11 11:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0008_metadata_expose"),
    ]

    operations = [
        migrations.AlterField(
            model_name="metadatatype",
            name="title",
            field=models.CharField(max_length=128, unique=True, verbose_name="titre"),
        ),
    ]



# Generated by Django 4.0 on 2022-01-31 16:36

from django.db import migrations

# from resources.models import MetadataSet, Metadata, Project


def add_fixtures(apps, schema_editor):
    """
    Adds region and rotation to iiif metadataset
    """
    Project = apps.get_model("resources", "Project")
    MetadataSet = apps.get_model("resources", "MetadataSet")
    Metadata = apps.get_model("resources", "Metadata")
    for project_instance in Project.objects.all():
        m_set, _ = MetadataSet.objects.get_or_create(
            project=project_instance, title="IIIF"
        )
        meta_region, _ = Metadata.objects.get_or_create(
            set=m_set, title="region", project=project_instance
        )
        meta_rotation, _ = Metadata.objects.get_or_create(
            set=m_set, title="rotation", project=project_instance
        )


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0026_project_props_fixtures"),
    ]

    # operations = [migrations.RunPython(add_fixtures)]
